// AI.cpp : Defines the entry point for the console application.
//

// --------------------------------------------------------------------------
// A Neural Net implementation by Craig Furness. Written early 2001.
// --------------------------------------------------------------------------
//
// After having written a working neural network, I tried to take a more OO
// approach. This is an attempt. It hasnt been too successful so far as good
// design goes, but it works.
//
// The math 'library' was an idea which stemmed from an approach of sharing
// mathematical abilities and functions (such as a graph) so that further
// AI projects would be made easier. The math library is IMHO a disaster.
// I cant explain why, but I do not like my approach to it.
// It has a very C-like feel to it, which I was hoping to get away from when
// I started this project. I have stripped out much of the math library code
// as the Neural Nets only use the POINT2D struct from it.
//
//
// Almost all of this project was written by myself, with the exception of the
// formula used to generate the output of each neuron. I copied that from some-
// where. I dont recall where.
//
// You are free to modify, use, do whatever with this code. 
//
// Some explanations:
//
// Learning Co-efficient : It is multiplied with the delta value for error
//                         to reduce or increase the size of weight changes
//                         for the links between neurons.
//
// Learning Decay : This is the rate of decay of the Learning Co-efficient
//                  between batches of training sets. Very useful for 'tweaking'
//                  the Neural Net! Value of 1.0 means no decay.
//
// Global Bias : A value which acts as a constant input to all neurons on the net.
//               The Neural Net can 'learn' to either use this value, or dis-
//               regard it. It *can* help the net to learn faster in some 
//               situations.
//
// Layer Bias : Similar to Global Bias, but for a particular layer only.
//
// Neuron Bias : Bias that is set only for a particular Neuron.
//
// Connection Model : Current supported is CNeuralNet::ALL which means every
//                    layer is fully interconnected with the layer above. Some
//                    Nets may function better if the layers are not fully
//                    connected. This is also possible if you either code in
//                    another connection model, or connect the neurons sep-
//                    erately using MakeConnection().
//
// --------------------------------------------------------------------------
// CNeuralNet provides some general functionality that most neural networks
// would use, and can be subclassed to implement specific types of Neural Networks. 
//
// CBPNet is a subclass of CNeuralNet and takes all of the 'generic' function-
// ality, and in addition implements functionality specific to a standard
// feed-forward network learning by back-propogation.
//
// The ability to feed-forward is interesting. For some reason I thought that
// this would be a nice 'module' to be able to plug-in to any net that needs it,
// as not all types of neural net would use the standard feed-forward algorithm.
// So the feed-forward bit is actually done using two #define's in feedforward.h
// All any neural net needs to do to get that functionality is include the line:
// PROTOTYPE_FEEDFORWARD
// in its class prototype (usually in the .h file), and then to implement the
// functionality prototyped by the above line, write this into the source file:
// IMPLEMENT_FEEDFORWARD(x)
// where x is the name of the class for which it is being implemented.
// This idea is kinda stolen from some Micro$oft things I have seen :)
// --------------------------------------------------------------------------
//
// Yeah. Have fun! TestNet() creates a 3 layer network and usually learns the
// XOR problem. You can create more than 3-layer networks.
//
// NOTE: This project has been written with a tab-size of 8. If yours is anything
// other than 8, some of the formatting could look strange.
//
// enjoy,
//
// Craig Furness (genix@cryo-genix.net)
//

#include "stdafx.h"
#include "iostream.h"

// neural net includes
#include "graph.h"
#include "math.h"
#include "time.h"
#include "BPNet.h"

void TestNet()
{
	// We need to set a random number seed to get more believable randomness :)
	srand( time(NULL) );

	// Create a 3-layer neural net to solve the XOR problem, with 2 nodes in the first two layers,
	// and a single node in the output layer.
	CBPNet XOR( 3 /* num layers */, 2 /* inputs */,2 /* hidden */,1 /* outputs */ );

	// connect the neurons up
	//
	//           O        - Output
	//          / \
	//         O   O      - Hidden
	//         |\ /|
	//         | X |
	//         |/ \|
	//         O   O      - Input
	//
	XOR.ConnectNeurons( CNeuralNet::ALL );

	// randomise the weightings
	XOR.Randomise();

	// set a global bias
	XOR.SetGlobalBias( 1 );

	// apply the bias to the hidden layer, and output layer
	XOR.SetBias( POINT2D(0,1),BIAS_GLOBAL );
	XOR.SetBias( POINT2D(1,1),BIAS_GLOBAL );
	XOR.SetBias( POINT2D(0,2),BIAS_GLOBAL );

	// start off with a reasonably high learning coefficient
	XOR.SetLearningCoefficient(1.5);

	// set the learning coefficient decay to be 0.999
	// for 4 training maps
	XOR.SetNumInputSets(4);
	XOR.SetLearningDecay(0.999);

	// train the network
	for( int ctr=0;ctr<2000;ctr++ )
	{
		// train for case where inputs are 0,0 and output is 0
		XOR.SetInputValue( 0,0 );
		XOR.SetInputValue( 1,0 );
		XOR.FeedForward();
		XOR.SetOutputValue( 0,0 );
		XOR.Train();
		// train for case where inputs are 0,1 and output is 1
		XOR.SetInputValue( 0,0 );
		XOR.SetInputValue( 1,1 );
		XOR.FeedForward();
		XOR.SetOutputValue( 0,1 );
		XOR.Train();
		// train for case where inputs are 1,0 and output is 1
		XOR.SetInputValue( 0,1 );
		XOR.SetInputValue( 1,0 );
		XOR.FeedForward();
		XOR.SetOutputValue( 0,1 );
		XOR.Train();
		// train for case where inputs are 1,1 and output is 0
		XOR.SetInputValue( 0,1 );
		XOR.SetInputValue( 1,1 );
		XOR.FeedForward();
		XOR.SetOutputValue( 0,0 );
		XOR.Train();
	}

	// run the network and report its output
	XOR.SetInputValue( 0,0 );
	XOR.SetInputValue( 1,0 );
	XOR.FeedForward();
	cout << "Input 0 0\tExpected 0\tOutput " << XOR.GetOutputValue(0) << endl;

	XOR.SetInputValue( 0,1 );
	XOR.SetInputValue( 1,0 );
	XOR.FeedForward();
	cout << "Input 1 0\tExpected 1\tOutput " << XOR.GetOutputValue(0) << endl;

	XOR.SetInputValue( 0,0 );
	XOR.SetInputValue( 1,1 );
	XOR.FeedForward();
	cout << "Input 0 1\tExpected 1\tOutput " << XOR.GetOutputValue(0) << endl;

	XOR.SetInputValue( 0,1 );
	XOR.SetInputValue( 1,1 );
	XOR.FeedForward();
	cout << "Input 1 1\tExpected 0\tOutput " << XOR.GetOutputValue(0) << endl;

	cout << "\nFinal learning coefficient was " << XOR.GetLearningCoefficient() << endl;
}

int main(int argc, char* argv[])
{
	TestNet();
	return 0;
}

