#include "stdafx.h"
#include "BPNet.h"

CBPNet::CBPNet() : CNeuralNet()
{
	m_LearningCoeff = 0.5;

	m_NumInputSets=-1;
	m_CurrentInputSet = 0;
	m_LearningDecay=1;
}

CBPNet::CBPNet( int p_NumLayers, ... ) : CNeuralNet( p_NumLayers, (void*)&p_NumLayers )
{
	m_LearningCoeff = 0.5;

	m_NumInputSets=-1;
	m_CurrentInputSet = 0;
	m_LearningDecay=1;
}

CBPNet::~CBPNet()
{
}

// Implement the feedforward routine for this class
IMPLEMENT_FEEDFORWARD(CBPNet)

inline double CBPNet::Formula( double p_val )
{
	return (1/(1+exp(-p_val)));
}

void CBPNet::Train()
{
	if (m_NumInputSets>0)
		if (m_CurrentInputSet++ == m_NumInputSets)
		{
			m_LearningCoeff*=m_LearningDecay;
			m_CurrentInputSet=0;
		}

	// calculate deltas for output layer
	for( int ctr1=m_NeuronCounts[m_NumLayers-1];ctr1>0;ctr1-- )
	{
		POINT2D NeuronPt = POINT2D(ctr1-1,m_NumLayers-1);
		LPNEURON Neuron = m_NeuralNet.GetNode( NeuronPt );
		Neuron->m_Delta = Neuron->m_Output * (1-Neuron->m_Output) * (Neuron->m_ExpectedOutput-Neuron->m_Output);
	}

	// calculate deltas for hidden layers based on layer above
	for( int ctr=m_NumLayers-2;ctr>0;ctr-- )
	{
		for ( ctr1=m_NeuronCounts[ctr];ctr1>0;ctr1-- )
		{
			POINT2D NeuronPt = POINT2D(ctr1-1,ctr);
			LPNEURON Neuron = m_NeuralNet.GetNode( NeuronPt );

			Neuron->m_Delta = Neuron->m_Output * (1-Neuron->m_Output) ;

			double UpperDeltas = 0,numUpperDeltas = 0;

			// find all connections from this node forward
			POINT2D To = m_NeuralNet.GetFirstLink( Neuron );
			while( !NO_VALUE(To) )
			{
				LPNEURON Dest = m_NeuralNet.GetNode( To );
				UpperDeltas+=(Dest->m_Weights[NeuronPt]*Dest->m_Delta);
				numUpperDeltas++;
				To = m_NeuralNet.GetNextLink();
			}

			Neuron->m_Delta*=(UpperDeltas/numUpperDeltas);
		}
	}

	// do weight changes
	for( ctr=m_NumLayers-1;ctr>0;ctr-- )
	{
		for ( int ctr1=m_NeuronCounts[ctr];ctr1>0;ctr1-- )
		{
			POINT2D NeuronPt = POINT2D(ctr1-1,ctr);
			LPNEURON Neuron = m_NeuralNet.GetNode( NeuronPt );

			// for each incoming connection, modify weight according to delta
			std::map<POINT2D,double>::iterator i;

			for( i=Neuron->m_Weights.begin();i!=Neuron->m_Weights.end();i++ )
				(*i).second+=(Neuron->m_Delta*m_LearningCoeff*Neuron->m_IncomingValues[(*i).first]);

			// adjust the bias weight if we are using it
			double BiasVal;

			switch( Neuron->m_BiasType )
			{
			case BIAS_GLOBAL: BiasVal = m_BiasGlobal;
				break;
			case BIAS_LAYER: BiasVal = m_LayerBias[ctr];
				break;
			case BIAS_NEURON: BiasVal = Neuron->m_Bias;
				break;
			case BIAS_NONE:;
			}

			if (Neuron->m_BiasType!=BIAS_NONE)
				Neuron->m_BiasWeight+=(Neuron->m_Delta*m_LearningCoeff*BiasVal);
		}
	}
}
