// CBPNet
//
// This class implements a specific type of neural network commonly called a Feed-forward Back-propogation
// Neural Network. It implements a basic verion of the delta rule to calculate error, and then backwardly
// propogates the error to determine weight changes coming into each node.
//
//
// -- Craig Furness. 2001

#ifndef BACKPROPNET

#define BACKPROPNET

#include "neuralnet.h"
#include "feedforward.h"

class CBPNet : public CNeuralNet
{
public:
	CBPNet();
	CBPNet( int p_NumLayers, ... );
	~CBPNet();

	// Back-propogation requires the ability to feedforward
	// so prototype it here.
	PROTOTYPE_FEEDFORWARD

	void	Train();

	// formula for calculating the input values to a neuron once the weights have been applied
	// usually sigmoid function
	double	Formula( double p_val );

	void	SetLearningCoefficient( double p_Coeff )	{ m_LearningCoeff = p_Coeff; }
	double	GetLearningCoefficient()			{ return m_LearningCoeff; }
	void	SetNumInputSets( int p_Input )			{ m_NumInputSets = p_Input; m_CurrentInputSet = 0; }
	void	SetLearningDecay( double p_Decay )		{ m_LearningDecay = p_Decay; }

private:
	double	m_LearningCoeff,m_LearningDecay;

	int m_NumInputSets,m_CurrentInputSet;
};

#endif