#ifndef G_GRAPH

#define G_GRAPH

#include "algebra.h"
#include <list>
#include <map>

typedef struct SNODE2D
{
	POINT2D m_Pos;
	std::list<SNODE2D*> m_OutConnections;
	std::list<SNODE2D*> m_InConnections;
} NODE2D, *LPNODE2D;

template< class T >
class CGraph2D
{
public:
	CGraph2D();
	~CGraph2D();

	void	AddSegment( POINT2D p_Start, POINT2D p_Stop );
	void	RemoveSegment( POINT2D p_Start, POINT2D p_Stop );
	void	RemoveAllSegments();
	T*	AddNode( POINT2D p_pos );
	void	RemoveNode( POINT2D p_pos );
	bool	HaveNode( POINT2D p_pos );
	bool	MoveNode( POINT2D p_current, POINT2D p_Dest );

	// get the first link leading off node p_Start
	// and the next
	// no more links return is when POINT2D has NO_VALUE()
	POINT2D GetFirstLink( T* p_Start );
	POINT2D GetFirstLink( POINT2D p_Start );
	POINT2D GetNextLink();

	// NOTE: this function returns a pointer to an actual node. modify returned data with caution
	T* GetNode( POINT2D p_pos )	{ return m_Nodes[p_pos]; }

private:
	std::map<POINT2D,T*> m_Nodes;

	std::list<LPNODE2D>::iterator m_GetLink;
	LPNODE2D m_GetLinkNode;
};

template< class T >
CGraph2D<T>::CGraph2D()
{
}

template< class T >
CGraph2D<T>::~CGraph2D()
{
}

template< class T >
void CGraph2D<T>::AddSegment( POINT2D p_Start, POINT2D p_Stop )
{
	T *Start = m_Nodes[p_Start], *Stop = m_Nodes[p_Stop];

	Start->m_OutConnections.push_back( Stop );
	Stop->m_InConnections.push_back( Start );
}

template< class T >
void CGraph2D<T>::RemoveSegment( POINT2D p_Start, POINT2D p_Stop )
{
	T *Start = m_Nodes[p_Start], *Stop = m_Nodes[p_Stop];

	Start->m_OutConnections.remove( Stop );
	Stop->m_InConnections.remove( Start );
}

template< class T >
void CGraph2D<T>::RemoveAllSegments()
{
	std::map<POINT2D,T*>::iterator i;

	for( i=m_Nodes.begin();i!=m_Nodes.end();i++ )
	{
		LPNODE2D Node = (*i).second;
		// remove all incoming
		Node->m_InConnections.clear();
		// remove all outgoing
		Node->m_OutConnections.clear();
	}
}

template< class T >
T* CGraph2D<T>::AddNode( POINT2D p_pos )
{
	T* Node = new T;
	Node->m_Pos = p_pos;
	m_Nodes[p_pos] = Node;
	return Node;
}

template< class T >
void CGraph2D<T>::RemoveNode( POINT2D p_pos )
{
	T *Node,*Other;

	Node = m_Nodes[p_pos];
	
	std::list<LPNODE2D>::iterator i;

	// remove all connections to this node
	for( i=Node->m_InConnections.begin();i!=Node->m_InConnections.end();i++ )
	{
		Other = m_Nodes[(*i)->m_Pos];

		std::list<LPNODE2D>::iterator j;
		
		for( j = Other->m_OutConnections.begin();j!=Other->m_OutConnections.end();j++ )
			if ((*j)->m_Pos==p_pos) break;

		Other->m_OutConnections.erase(j);
	}

	for( i=Node->m_OutConnections.begin();i!=Node->m_OutConnections.end();i++ )
	{
		Other = m_Nodes[(*i)->m_Pos];

		std::list<LPNODE2D>::iterator j;

		for( j = Other->m_InConnections.begin();j!=Other->m_InConnections.end();j++ )
			if ((*j)->m_Pos==p_pos) break;

		Other->m_OutConnections.erase(j);
	}

	// erase this node
	m_Nodes.erase( p_pos );

	delete Node;
}

template< class T >
POINT2D CGraph2D<T>::GetFirstLink( T* p_Start )
{
	m_GetLinkNode = p_Start;
	m_GetLink = p_Start->m_OutConnections.begin();
	if (m_GetLink!=p_Start->m_OutConnections.end()) return (*m_GetLink)->m_Pos;
	return POINT2D(NO_VALUE(),NO_VALUE());
}

template< class T >
POINT2D CGraph2D<T>::GetFirstLink( POINT2D p_Start )
{
	T* Node = m_Nodes[p_Start];
	m_GetLinkNode = Node;
	m_GetLink = Node->m_OutConnections.begin();
	if (m_GetLink!=Node->m_OutConnections.end()) return (*m_GetLink)->m_Pos;
	return POINT2D(NO_VALUE(),NO_VALUE());
}

template< class T >
POINT2D CGraph2D<T>::GetNextLink()
{
	m_GetLink++;

	if (m_GetLink!=m_GetLinkNode->m_OutConnections.end())
		return (*m_GetLink)->m_Pos;

	return POINT2D(NO_VALUE(),NO_VALUE());
}

template< class T >
bool CGraph2D<T>::HaveNode( POINT2D p_pos )
{
	std::map<POINT2D,T*>::iterator i = m_Nodes.find( p_pos );
	if (i!=m_Nodes.end()) return true;
	return false;
}

template< class T >
bool CGraph2D<T>::MoveNode( POINT2D p_current, POINT2D p_Dest )
{
	if (HaveNode(p_Dest)) return false;

	T *Node = m_Nodes[p_current],*Other;

	// adjust all links to node to reflect new position
	std::list<LPNODE2D>::iterator i;

	for( i=Node->m_InConnections.begin();i!=Node->m_InConnections.end();i++ )
	{
		Other = m_Nodes[(*i)->m_Pos];

		std::list<LPNODE2D>::iterator j;
		
		for( j = Other->m_OutConnections.begin();j!=Other->m_OutConnections.end();j++ )
			if ((*j)->m_Pos==p_current) break;

		(*j)->m_Pos = p_Dest;
	}

	for( i=Node->m_OutConnections.begin();i!=Node->m_OutConnections.end();i++ )
	{
		Other = m_Nodes[(*i)->m_Pos];

		std::list<LPNODE2D>::iterator j;

		for( j = Other->m_InConnections.begin();j!=Other->m_InConnections.end();j++ )
			if ((*j)->m_Pos==p_current) break;

		(*j)->m_Pos = p_Dest;
	}

	Node->m_Pos = p_Dest;

	return true;
}

#endif