#include "stdafx.h"
#include "neuralnet.h"
#include <cstdarg>

#include "iostream.h"

CNeuralNet::CNeuralNet()
{
}

CNeuralNet::CNeuralNet( int p_NumLayers, void* p_vargs )
{
	va_list lstneurons;
	va_start( lstneurons, *((int*)p_vargs) );
	m_NumLayers = p_NumLayers; 
	m_NeuronCounts.resize( m_NumLayers );
	m_LayerBias.resize( m_NumLayers );
	for( int ctr=0; ctr<m_NumLayers; ctr++ )
	{
		int numneurons = va_arg( lstneurons,int );
		for( int ctr2=numneurons;ctr2>0;ctr2-- )
			m_NeuralNet.AddNode( POINT2D(ctr2-1,ctr) );
		m_NeuronCounts[ctr] = numneurons;
	}
	va_end( lstneurons );
}

CNeuralNet::~CNeuralNet()
{
}

void CNeuralNet::ConnectNeurons( LINKMODEL p_LinkModel, int p_StartLayer, int p_EndLayer )
{
	int Start = p_StartLayer, End = ((p_EndLayer==-1)?m_NumLayers-1:p_EndLayer);
	int ctr,ctr1,ctr2;

	switch( p_LinkModel )
	{
	case ALL:
		for( ctr=Start;ctr<End;ctr++ )
			for( ctr1=0;ctr1<m_NeuronCounts[ctr];ctr1++ )
				for( ctr2=0;ctr2<m_NeuronCounts[ctr+1];ctr2++ )
				{
					LPNEURON Neuron = m_NeuralNet.GetNode( POINT2D(ctr2,ctr+1) );

					// add segment to the graph
					m_NeuralNet.AddSegment( POINT2D(ctr1,ctr),POINT2D(ctr2,ctr+1) );

					// add details to our neuron
					Neuron->m_IncomingValues[POINT2D(ctr1,ctr)] = 0;
					Neuron->m_Weights[POINT2D(ctr1,ctr)] = 0;
				}
		break;
	};
}

void CNeuralNet::RemoveNeuron( POINT2D p_Neuron )
{
	m_NeuralNet.RemoveNode( p_Neuron );

	// for each neuron after this, shift its position down
	for( int ctr=p_Neuron.m_xPos+1;ctr<m_NeuronCounts[p_Neuron.m_yPos];ctr++ )
	{
		POINT2D Neuron( ctr,p_Neuron.m_yPos ),Dest( ctr-1,p_Neuron.m_yPos );
		m_NeuralNet.MoveNode( Neuron, Dest );
	}

	m_NeuronCounts[p_Neuron.m_yPos]--;

	// find all connections linking to neurons
	// and shift them down
	for( ctr=0;ctr<m_NumLayers-1;ctr++ )
		for( int ctr1=0;ctr1<m_NeuronCounts[ctr];ctr1++ )
		{
			LPNEURON Neuron = m_NeuralNet.GetNode( POINT2D(ctr1,ctr) );

			std::map<POINT2D,double>::iterator iw;
			for( iw=Neuron->m_Weights.begin();iw!=Neuron->m_Weights.end();iw++ )
			{
				if ((*iw).first.m_yPos==p_Neuron.m_yPos)
				{
					if ((*iw).first.m_xPos>p_Neuron.m_xPos)
					{
						Neuron->m_Weights[ POINT2D((*iw).first.m_xPos-1,(*iw).first.m_yPos) ]=(*iw).second;
						iw = Neuron->m_Weights.erase( iw );
					}
					else
						if ((*iw).first==p_Neuron) iw = Neuron->m_Weights.erase( iw );
				}
			}

			std::map<POINT2D,double>::iterator iv;
			for( iv=Neuron->m_IncomingValues.begin();iv!=Neuron->m_IncomingValues.end();iv++ )
			{
				if ((*iv).first.m_yPos==p_Neuron.m_yPos)
				{
					if ((*iv).first.m_xPos>p_Neuron.m_xPos)
					{
						Neuron->m_Weights[ POINT2D((*iv).first.m_xPos-1,(*iv).first.m_yPos) ]=(*iv).second;
						iv = Neuron->m_IncomingValues.erase( iv );
					}
					else
						if ((*iv).first==p_Neuron) iv = Neuron->m_IncomingValues.erase( iv );
				}
			}
		}
}

POINT2D CNeuralNet::AddNeuron( int p_Layer )
{
	POINT2D ret(m_NeuronCounts[p_Layer]++,p_Layer);
	LPNEURON Neuron = m_NeuralNet.AddNode( ret );
	Neuron->m_BiasType = BIAS_NONE;
	return ret;
}

void CNeuralNet::RemoveConnections()
{
	for( int ctr=0;ctr<m_NumLayers-1;ctr++ )
		for( int ctr1=0;ctr1<m_NeuronCounts[ctr];ctr1++ )
			for( int ctr2=0;ctr2<m_NeuronCounts[ctr+1];ctr2++ )
			{
				LPNEURON Neuron = m_NeuralNet.GetNode( POINT2D(ctr2,ctr+1) );

				Neuron->m_IncomingValues.clear();
				Neuron->m_Weights.clear();
			}
}

void CNeuralNet::RemoveConnection( POINT2D p_Start, POINT2D p_End )
{
	m_NeuralNet.RemoveSegment( p_Start, p_End );

	LPNEURON Neuron = m_NeuralNet.GetNode( p_End );

	Neuron->m_IncomingValues.erase( p_Start);
	Neuron->m_Weights.erase( p_Start );
}

void CNeuralNet::MakeConnection( POINT2D p_Start, POINT2D p_End )
{
	m_NeuralNet.AddSegment( p_Start, p_End );

	LPNEURON Neuron = m_NeuralNet.GetNode( p_End );

	Neuron->m_IncomingValues[p_Start] = 0;
	Neuron->m_Weights[p_Start] = 0;
}

double CNeuralNet::GetWeight( POINT2D p_Start, POINT2D p_End )
{
	LPNEURON Neuron = m_NeuralNet.GetNode( p_End );
	return Neuron->m_Weights[ p_Start ];
}

void CNeuralNet::SetWeight( POINT2D p_Start, POINT2D p_End, double p_Weight )
{
	LPNEURON Neuron = m_NeuralNet.GetNode( p_End );
	Neuron->m_Weights[ p_Start ] = p_Weight;
}

void CNeuralNet::Randomise()
{
	for( int ctr=1;ctr<m_NumLayers;ctr++ )
	{
		for( int ctr1=0;ctr1<m_NeuronCounts[ctr];ctr1++ )
		{
			// get ptr to our own struct
			LPNEURON Neuron = m_NeuralNet.GetNode( POINT2D(ctr1,ctr) );
			std::map<POINT2D,double>::iterator i;
			for( i=Neuron->m_Weights.begin();i!=Neuron->m_Weights.end();i++ )
				(*i).second = ((double)(rand()%10000) / 5000)-1;
			Neuron->m_BiasWeight = ((double)(rand()%10000) / 5000)-1;
		}
	}
}

void CNeuralNet::SetInputValue( int p_num, double p_val )
{
	POINT2D thispoint(p_num,0);
	LPNEURON Neuron = m_NeuralNet.GetNode(thispoint);
	Neuron->m_Output = p_val;
}

double CNeuralNet::GetOutputValue( int p_num )
{
	LPNEURON Neuron = m_NeuralNet.GetNode( POINT2D(p_num,m_NumLayers-1) );
	return Neuron->m_Output;
}

void CNeuralNet::SetOutputValue( int p_num, double p_val )
{
	LPNEURON Neuron = m_NeuralNet.GetNode( POINT2D(p_num,m_NumLayers-1) );
	Neuron->m_ExpectedOutput = p_val;
}

void CNeuralNet::SetBias( POINT2D p_Neuron, BIAS_TYPE p_Bias )
{
	LPNEURON Neuron = m_NeuralNet.GetNode( p_Neuron );
	Neuron->m_BiasType = p_Bias;
}
