// CNeuralNet
//
// Base class implements all the functionality which is common between various types of neural networks.
// The Neural Net is made from NEURON's which are derived from NODE2D, which are in turn used by CGraph2D
// to construct a directional 2d graph.
//
// This is intended to be an abstract class, and instances should rather be made of specific types of ANN's.
//
//
// -- Craig Furness. 2001

#ifndef NEURALNET

#define NEURALNET

#include "graph.h"
#include <cstdarg>
#include <map>
#include <vector>

enum BIAS_TYPE { BIAS_NONE = 0, BIAS_NEURON, BIAS_LAYER, BIAS_GLOBAL };

typedef struct SNEURON : NODE2D
{
	double m_Output, m_Delta, m_ExpectedOutput;
	std::map<POINT2D,double> m_Weights;		// weights for all incoming connections
	std::map<POINT2D,double> m_IncomingValues;	// prior to modification by weights

	BIAS_TYPE m_BiasType;
	double m_Bias,m_BiasWeight;
} NEURON, *LPNEURON;

class CNeuralNet
{
public:
	enum LINKMODEL { ALL = 0 };

	CNeuralNet();
	CNeuralNet( int p_NumLayers, void* p_vargs );
	~CNeuralNet();

	void	ConnectNeurons		( LINKMODEL, int p_StartLayer = 0, int p_EndLayer = -1 );
	void	RemoveConnections	();
	void	RemoveConnection	( POINT2D p_Start, POINT2D p_End );
	void	MakeConnection		( POINT2D p_Start, POINT2D p_End );
	double	GetWeight		( POINT2D p_Start, POINT2D p_End );
	void	SetWeight		( POINT2D p_Start, POINT2D p_End, double p_Weight );
	void	Randomise();
	void	RemoveNeuron		( POINT2D p_Neuron );	// removes neuron at this point, shifting others in same row down
	POINT2D	AddNeuron		( int p_Layer );	// adds neuron to this layer, returns neuron location

	void	SetBias			( POINT2D p_Neuron, BIAS_TYPE p_Bias ); // sets bias of specified type on neuron
	void	SetGlobalBias		( double p_val )			{ m_BiasGlobal = p_val; }
	void	SetLayerBias		( int p_layernum, double p_val )	{ m_LayerBias[p_layernum] = p_val; }
	void	SetNeuronBias		( POINT2D p_Neuron, double p_val )	{ LPNEURON Neuron = m_NeuralNet.GetNode(p_Neuron); Neuron->m_Bias = p_val; }

	void	SetInputValue( int p_num, double p_val );
	double	GetOutputValue( int p_num );
	void	SetOutputValue( int p_num, double p_val );

protected:
	CGraph2D<NEURON> m_NeuralNet;

	int m_NumLayers;				// number of layers in the network (incl. input and output layers)
	std::vector<int> m_NeuronCounts;		// number of neurons per layer
	std::vector<double> m_LayerBias;		// stores the bias for each layer

	double m_BiasGlobal;				// stores the global bias

private:
};

#endif